<?php

namespace Drupal\modeler_api\Entity;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\modeler_api\Api;

/**
 * Defines the access control handler for model owner config entities.
 */
class AccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account): AccessResultInterface {
    if ($operation === 'update' && $entity instanceof ConfigEntityInterface) {
      $owner = Api::get()->findOwner($entity);
      if ($owner && $owner->isEditable($entity) && $owner->getModelData($entity) && $modeler = $owner->getModeler($entity)) {
        return AccessResult::allowedIf($modeler->isEditable());
      }
      return AccessResult::forbidden();
    }

    return parent::checkAccess($entity, $operation, $account);
  }

}
